@@ -498,7 +498,7 @@ read_config_file (const char *filename, gboolean enoent_error)
 	{
 	  gboolean empty = FALSE;
 	  gboolean append = FALSE;
-	  char *k, *v;
+	  char *v;
 
 	  if (!section)
 	    {
@@ -545,16 +545,12 @@ read_config_file (const char *filename, gboolean enoent_error)
 	  g_string_prepend_c (tmp_buffer1, '/');
 	  g_string_prepend (tmp_buffer1, section);
 
-	  /* Remove any existing values */
-	  if (g_hash_table_lookup_extended (config_hash, tmp_buffer1->str,
-					    (gpointer *)&k, (gpointer *)&v))
+	  if (append)
 	    {
-	      g_free (k);
-	      if (append)
-		{
-		  g_string_prepend (tmp_buffer2, v);
-		  g_free (v);
-		}
+	      /* Get any existing value */
+	      v = g_hash_table_lookup (config_hash, tmp_buffer1->str);
+	      if (v)
+		g_string_prepend (tmp_buffer2, v);
 	    }
 	      
 	  if (!empty)
@@ -594,7 +590,9 @@ read_config ()
       const char *home;
       const char *envvar;
       
-      config_hash = g_hash_table_new (g_str_hash, g_str_equal);
+      config_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+					   (GDestroyNotify)g_free,
+					   (GDestroyNotify)g_free);
       filename = g_build_filename (pango_get_sysconf_subdirectory (),
 				   "pangorc",
 				   NULL);
@@ -1179,8 +1177,8 @@ static LangInfo lang_texts[] = {
   { "fi", "Finnish (Suomi)  Hei, Hyv\303\244\303\244 p\303\244iv\303\244\303\244" },
   { "fr", "French (Fran\303\247ais)" },
   { "de", "German Gr\303\274\303\237 Gott" },
-  { "iw", "Hebrew   \327\251\327\234\327\225\327\235" },
-  { "il", "Italiano  Ciao, Buon giorno" },
+  { "he", "Hebrew   \327\251\327\234\327\225\327\235" },
+  { "it", "Italiano  Ciao, Buon giorno" },
   { "ja", "Japanese (\346\227\245\346\234\254\350\252\236) \343\201\223\343\202\223\343\201\253\343\201\241\343\201\257, \357\275\272\357\276\235\357\276\206\357\276\201\357\276\212" },
   { "ko", "Korean (\355\225\234\352\270\200)   \354\225\210\353\205\225\355\225\230\354\204\270\354\232\224, \354\225\210\353\205\225\355\225\230\354\213\255\353\213\210\352\271\214" },
   { "mt", "Maltese   \304\212aw, Sa\304\247\304\247a" },
@@ -1255,16 +1253,15 @@ pango_log2vis_get_embedding_levels (gunichar       *str,
   return result;
 }
 
+#endif /* HAVE_FRIBIDI */
 
 gboolean 
 pango_get_mirror_char (gunichar        ch,
 		       gunichar       *mirrored_ch)
 {
-  return fribidi_get_mirror_char (ch, mirrored_ch); 
+  return g_unichar_get_mirror_char (ch, mirrored_ch); 
 }
 
-#endif /* HAVE_FRIBIDI */
-
 
 static guint
 alias_hash (struct PangoAlias *alias)
